"""
  The moment of inertia is like the mass of an object, but applied 
  to its rotation. An object with a higher moment of inertia is harder to spin.
"""

import math
import pygame
from pygame.locals import *
from pygame.color import *

import pymunk as pm
from pymunk.vec2d import vec2d

def flipy(y):
	"""Small hack to convert chipmunk physics to pygame coordinates"""
	return -y+600

def add_static_L(space):
	rotation_center_body = pm.Body(pm.inf, pm.inf)
	rotation_center_body.position = vec2d(300,300)
	
	rotation_limit_body = pm.Body(pm.inf, pm.inf)
	rotation_limit_body.position = vec2d(200,300)
	
	body = pm.Body(10, 10000)
	body.position = vec2d(300,300)
	l1 = pm.Segment(body, vec2d(-150, 0), vec2d(255.0, 0.0), 5.0)
	l1.friction = 0.8
	
	l2 = pm.Segment(body, vec2d(-150.0, 0), vec2d(-150.0, 50.0), 5.0)
	l2.friction = 2.0
	
	rotation_center_joint = pm.PinJoint(body, rotation_center_body, vec2d(0,0), vec2d(0,0)) 
	joint_limit = 25
	rotation_limit_joint = pm.SlideJoint(body, rotation_limit_body, vec2d(-100,0), vec2d(0,0), 0, joint_limit) 

	space.add(l1, l2, body, rotation_center_joint, rotation_limit_joint)
	return l1,l2
	
def add_square(space, pos):
	# Square Vectors (Clockwise)
        square_verts = [vec2d(-18,-18), vec2d(-18, 18), vec2d(18, 18), vec2d(18,-18)]
	
	# Square Physic Settings
        square_mass = 5.0
        square_moment = pm.moment_for_poly(square_mass, square_verts, vec2d(0,0))
        square_friction = 0.2

	# Create Body
        square = pm.Body(square_mass, square_moment)
        square.position = vec2d(pos)

	# Create Shape
        shape = pm.Poly(square, square_verts, vec2d(0,0))
        shape.friction = square_friction
        
        # Append to Space
        space.add(shape, square)
        
        # Return Shape
        return shape
        
def add_ball(space, pos):
	# Ball Physic Settings
	ball_mass = 10.0
	ball_intertia = 1000
	ball_friction = 2.0
	
	# Create Body
	body = pm.Body(ball_mass, ball_intertia)
	body.position = vec2d(pos)
	
	# Create Shape
	shape = pm.Circle(body, 20, vec2d(0,0))
	shape.friction = ball_friction
	
	# Append to Space
	space.add(body, shape)
	
	# Return Shape
	return shape

def draw_poly(screen, poly, closed=True):
	# Correct Poly Coordinates
	points = []
	for p in poly.get_points():
		points.append((p[0], flipy(p[1])))

	# Finish the Polygon
	if closed and points[-1] != points[0]:
		points.append(points[0])

	# Draw Poly
	pygame.draw.lines(screen, THECOLORS["blue"], False, points, 2)

def draw_ball(screen, ball):
	# Get Ball Infos
	r = ball.radius
	v = ball.body.position
	rot = ball.body.rotation_vector
	
	# Draw Ball
	p = int(v.x), int(flipy(v.y))
	pygame.draw.circle(screen, THECOLORS["blue"], p, int(r), 2)

	# Draw Rotation Vector
	p2 = vec2d(rot.x, -rot.y) * r * 0.9
	pygame.draw.line(screen, THECOLORS["red"], p, p+p2)

def draw_lines(screen, lines):
	for line in lines:
		body = line.body
		pv1 = body.position + line.a.rotated(math.degrees(body.angle))
		pv2 = body.position + line.b.rotated(math.degrees(body.angle))
		p1 = pv1.x, flipy(pv1.y)
		p2 = pv2.x, flipy(pv2.y)
		pygame.draw.lines(screen, THECOLORS["blue"], False, [p1,p2])

def main():
	# PyGame Init
	pygame.init()
	screen = pygame.display.set_mode((600, 600))
	clock = pygame.time.Clock()

	# Default Settings
	running = True
	run_physics = True # for pause

	# Physics Init
	pm.init_pymunk()
	
	# Physics Init: Space
	space = pm.Space()
	space.gravity = vec2d(0.0, -900.0)
	space.resize_static_hash()
	space.resize_active_hash()
	
	# Elements
	balls = []
	lines = []
	squares = []
	polys = []

	shapes = add_static_L(space)
	lines.append(shapes)

	# Main Loop
	while running:
		for event in pygame.event.get():
			if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
				# Quit
				running = False
				
			elif event.type == KEYDOWN and event.key == K_SPACE:	
				# Pause Physics
				run_physics = not run_physics

			elif event.type == MOUSEBUTTONDOWN and event.button == 1:
				# Add Ball
				pos = event.pos[0], flipy(event.pos[1])
				ball_shape = add_ball(space, pos)
				balls.append(ball_shape)
				
			elif event.type == MOUSEBUTTONDOWN and event.button == 3:
				# Add a Poly
				pos = event.pos[0], flipy(event.pos[1])
				poly_shape = add_square(space, pos)
				polys.append(poly_shape)


		for b in space.get_bodies():
			b.reset_forces()
							
		# Update physics
		if run_physics:
			dt = 1.0/50.0
			space.step(dt)
			
		# Clear Display
		screen.fill((255,255,255))

		# Draw Elements
		for ball in balls:		   
			draw_ball(screen, ball)

		for l in lines:
			draw_lines(screen, l)

		for poly in polys:
			draw_poly(screen, poly)
						
		# Flip screen
		pygame.display.flip()
		
		# keep 50 fps steady
		clock.tick(50)
		
		# output framerate in caption
		pygame.display.set_caption("fps: " + str(clock.get_fps()))

if __name__ == "__main__":
	main()
