"""
	I. Interesting thing to mention about this demo is this part (@138):

		if run_physics:
			clk = clock.get_fps()
			if clk == 0.0: 
				clk = 50.0
			dt = 1.0/clk
	
	For example. if the framerate drops to 25fps due to many objects, the main 
	loop will runn only 25 times a second. Therefore,

	  1. if clk is set to 50.0:
	     -> One second in the physics space will take 2 'real' seconds
	     -> All 50 steps of the Elements for one second will be displayed
	     -> Slower with many objects

	  2. if clk is set to clock.get_fps():
 	     -> Less movements of the Elements if Framerate drops
	     -> Synced with 'real' seconds
	     -> Faster with many objects (>300)
	     

	     
	II. Removing balls which fall down (@149):

		# draw_ball returns False if ball falls down
		for ball in balls:		   
			if not draw_ball(screen, ball):
				balls.remove(ball)
				space.remove(ball)
	
"""
import math

import pygame
from pygame.locals import *
from pygame.color import *

import pymunk as pm
from pymunk.vec2d import vec2d

def flipy(y):
	"""Small hack to convert chipmunk physics to pygame coordinates"""
	return -y+900

def add_ball(space, pos):
	ball_radius = 7
	ball_mass = 100000.0
	ball_intertia = 0.5
	ball_friction = 0.9
	ball_elasticity = 0.5
		
	# Create Body
	body = pm.Body(ball_mass, ball_intertia)
	body.position = vec2d(pos)

	shape = pm.Circle(body, ball_radius, vec2d(0,0))
	shape.friction = ball_friction
	shape.set_elasticity = ball_elasticity
	space.add(body, shape)
	
	return shape

def draw_ball(screen, ball):
	r = ball.radius
	v = ball.body.position
	rot = ball.body.rotation_vector
	p = int(v.x), int(flipy(v.y))
	pygame.draw.circle(screen, THECOLORS["blue"], p, int(r), 2)

#	p2 = vec2d(rot.x, -rot.y) * r * 0.9
#	pygame.draw.line(screen, THECOLORS["red"], p, p+p2)

	if p[1] > 900: return False
	return True	

def add_wall(space, p1, p2):
	mass = 1e100 # unlimited
	inertia = 1e100 # unlimited
		
	body = pm.Body(mass, inertia)
	shape= pm.Segment(body, vec2d((p1[0], flipy(p1[1]))), vec2d((p2[0], flipy(p2[1]))), 2.0)	
	shape.friction = 10.0
	space.add(shape)
	
	# will be added as/to lines
	return [shape]

def draw_lines(screen, lines):
	for line in lines:
		body = line.body
		pv1 = body.position + line.a.rotated(math.degrees(body.angle))
		pv2 = body.position + line.b.rotated(math.degrees(body.angle))
		p1 = pv1.x, flipy(pv1.y)
		p2 = pv2.x, flipy(pv2.y)
		pygame.draw.lines(screen, THECOLORS["blue"], False, [p1,p2])

def main():
	# PyGame Init
	pygame.init()
	screen = pygame.display.set_mode((900, 900))
	clock = pygame.time.Clock()

	# Default Settings
	running = True
	run_physics = True # for pause

	# Physics Init
	pm.init_pymunk()
	
	# Physics Init: Space
	space = pm.Space()
	space.gravity = vec2d(0.0, -900.0)
        space.resize_static_hash(7.0, 2000)
        space.resize_active_hash(7.0, 100)
	
	# Elements
	balls = []
	lines = []
			
	# Make the wall
	lines.append(add_wall(space, (10,650), (100,800)))
	lines.append(add_wall(space, (100,800), (400,800)))
	lines.append(add_wall(space, (400,800), (425,770)))
	
	lines.append(add_wall(space, (475,770), (500,800)))
	lines.append(add_wall(space, (500,800), (800,800)))
	lines.append(add_wall(space, (800,800), (890,650)))

#	lines.append(add_wall(space, (10,10), (10,850)))
#	lines.append(add_wall(space, (10,850), (890,850)))
#	lines.append(add_wall(space, (890,850), (890,10)))
	
	# Main Loop
	while running:
		for event in pygame.event.get():
			if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
				# Quit
				running = False
				
			elif event.type == KEYDOWN and event.key == K_SPACE:	
				# Pause Physics
				run_physics = not run_physics

			elif event.type == MOUSEMOTION:
				# Add Ball with Left Mouse Button
				pos = event.pos[0], flipy(event.pos[1])
				ball_shape = add_ball(space, pos)
				balls.append(ball_shape)
				
		# Update physics
		if run_physics:
			clk = clock.get_fps()
			if clk == 0.0: 
				clk = 50.0
			dt = 1.0/clk
			space.step(dt)
			
		# Clear Display
		screen.fill((255,255,255))

		# Draw Elements
		for ball in balls:		   
			if not draw_ball(screen, ball):
				balls.remove(ball)
				space.remove(ball)

		for l in lines:
			draw_lines(screen, l)

		# Flip screen
		pygame.display.flip()
		
		# keep 50 fps steady
		clock.tick(50)
		
		# output framerate in caption
		pygame.display.set_caption("balls: %i | fps: %s " % (len(balls), str(clock.get_fps())))

if __name__ == "__main__":
	main()
